==> kxstitch_0.6.orig.tar.gz: kxstitch-0.6/kxstitch/importimagedlg.cpp <==
/***************************************************************************
 *   Copyright (C) 2003 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qpixmap.h>
#include <qpainter.h>
#include <qptrlist.h>
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qprogressdialog.h>
#include <qtoolbutton.h>
#include <knuminput.h>
#include <klineedit.h>
#include <kapplication.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include "configuration.h"
#include "importimagedlg.h"
#include "kxstitch.h"
#include "flossscheme.h"
#include "floss.h"

ImportImageDlg::ImportImageDlg(const Magick::Image& image, QWidget *parent, const char *name, bool modal)
  : ImportImage(parent,name,modal),
    m_pixmap(new QPixmap),
    m_orig(image)
{
  int w = m_orig.columns();
  int h = m_orig.rows();

  // get the users preferred pattern size in stitches
  double cc = KXSConfig().NewPattern_ClothCount;
  QString ccu = KXSConfig().NewPattern_ClothCountUnits;
  Import_ClothCount->setSuffix(ccu);
  Import_ClothCount->setRange(1,55,(ccu == "/cm")?0.01:1.0,false);
  Import_ClothCount->setValue(cc);
  int pw = KXSConfig().NewPattern_Width;
  int ph = KXSConfig().NewPattern_Height;
  int sw = pw*100/w;
  int sh = ph*100/h;
  int s = sw<?sh; // pick smallest scale

  Import_FlossScheme->insertStringList(((KXStitchApplication*)kapp)->schemes());
  Configuration::setComboBoxItem(Import_FlossScheme,KXSConfig().NewPattern_FlossScheme);
  createImageMap();
  Import_PatternScale->setValue(s);
  Import_UseMaxColors->setChecked(KXSConfig().Import_UseMaxColors);
  Import_MaxColors->setValue(KXSConfig().Import_MaxColors);

  calcSizes();

  connect(Import_FlossScheme, SIGNAL(activated(int)), this, SLOT(schemeChanged(int)));
  connect(Import_UseMaxColors, SIGNAL(stateChanged(int)), this, SLOT(maxColorsChanged(int)));
  connect(Import_MaxColors, SIGNAL(valueChanged(int)), this, SLOT(maxColorsChanged(int)));
  connect(Import_ClothCount, SIGNAL(valueChanged(double)), this, SLOT(clothCountChanged(double)));
  connect(Import_PatternScale, SIGNAL(valueChanged(int)), this, SLOT(scaleChanged(int)));
  connect(ResetButton, SIGNAL(clicked()), this, SLOT(resetImage()));
  connect(HelpButton, SIGNAL(clicked()), this, SLOT(contextHelp()));

  renderPixmap();
}

ImportImageDlg::~ImportImageDlg()
{
  delete m_pixmap;
}

void ImportImageDlg::resetImage()
{
  m_image = m_orig;
  calcSizes();
  renderPixmap();
}

void ImportImageDlg::schemeChanged(int)
{
  createImageMap();
  calcSizes();
  renderPixmap();
}

void ImportImageDlg::calcSizes()
{
  m_image = m_orig;
  Magick::Geometry geom(Import_PatternScale->value(), Import_PatternScale->value());
  geom.percent(true);
  m_image.sample(geom);
  int w = m_image.columns();
  int h = m_image.rows();
  m_size = QSize(w,h);
  clothCountChanged(Import_ClothCount->value());
}

void ImportImageDlg::maxColorsChanged(int)
{
  calcSizes();
  renderPixmap();
}

void ImportImageDlg::clothCountChanged(double n)
{
  double wi = m_size.width();
  double hi = m_size.height();
  QString suffix;
  QString units = KXSConfig().NewPattern_Units;
  if (units != "Stitches")
  {
    wi /= n;
    hi /= n;
    suffix = Import_ClothCount->suffix().right(2);
  }
  PatternSize->setText(QString(i18n("%1%2 x %3%4")).arg(wi,0,'g',3).arg(suffix).arg(hi,0,'g',3).arg(suffix));
}

void ImportImageDlg::scaleChanged(int)
{
  calcSizes();
  renderPixmap();
}

void ImportImageDlg::createImageMap()
{
  FlossScheme *scheme = ((KXStitchApplication*)kapp)->scheme(Import_FlossScheme->currentText());
  QPtrListIterator<Floss> it = scheme->flossIterator();
  char *pixels = new char[it.count()*3];
  char *pixel = pixels;
  Floss *f;
  for ( ; (f = it.current()) ; ++it)
  {
    *pixel++ = (char)(f->color.red());
    *pixel++ = (char)(f->color.green());
    *pixel++ = (char)(f->color.blue());
  }
  m_map = Magick::Image(it.count(), 1, "RGB", MagickLib::CharPixel, pixels);
  delete pixels;
}

void ImportImageDlg::renderPixmap()
{
  CropImageLabel->setCursor(waitCursor);
  m_image.modifyImage();
  m_pixmap->resize(m_size);
  m_pixmap->fill();
  if (Import_UseMaxColors->isChecked())
  {
    // TODO add noise reduction routines to clean up converted images
    m_image.quantizeColorSpace(Magick::RGBColorspace);
    m_image.quantizeColors(Import_MaxColors->value());
    m_image.quantize();
  }
  m_image.map(m_map);
  m_image.modifyImage();

  QPainter p;
  p.begin(m_pixmap);
  int w = m_size.width();
  int h = m_size.height();
  int pixelCount = w*h;

  QProgressDialog progress(i18n("Rendering preview..."), i18n("Cancel"), pixelCount, this, i18n("Progress"), true);
  progress.setMinimumDuration(100);

  Magick::Pixels cache(m_image);
  const Magick::PixelPacket *pixels = cache.get(0,0,w,h);
  for (int cy = 0 ; cy < h ; cy++)
  {
    kapp->processEvents();
    progress.setProgress(cy*w);
    if (progress.wasCancelled()) break;
    for (int cx = 0 ; cx < w ; cx++)
    {
      Magick::PixelPacket packet = *pixels++;
      Magick::Color c = Magick::Color(packet.red, packet.green, packet.blue);
      if (!(packet.opacity))
      {
        QColor c(packet.red/257, packet.green/257, packet.blue/257);
        p.setPen(QPen(c));
        p.drawPoint(cx,cy);
      }
    }
  }
  p.end();
  CropImageLabel->setPixmap(*m_pixmap);
  CropImageLabel->setCursor(arrowCursor);
}

void ImportImageDlg::accept()
{
  if (Import_SetDefault->isChecked())
  {
    KXSConfig().Import_UseMaxColors = Import_UseMaxColors->isChecked();
    KXSConfig().Import_MaxColors = Import_MaxColors->value();
    KXSConfig().writeSettings();
  }
  QDialog::accept();
}

void ImportImageDlg::contextHelp()
{
  kapp->invokeHelp("ImportImageDlg");
}

==> kxstitch_0.8.3.1.orig.tar.gz: kxstitch-0.8.3.1/kxstitch/importimagedlg.cpp <==
/***************************************************************************
 *   Copyright (C) 2003 by Stephen Allewell                                *
 *   stephen@mirramar.fsnet.co.uk                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <qpixmap.h>
#include <qpainter.h>
#include <qptrlist.h>
#include <qtabwidget.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qprogressdialog.h>
#include <qtoolbutton.h>
#include <knuminput.h>
#include <klineedit.h>
#include <kapplication.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <qevent.h>
#include "configuration.h"
#include "importimagedlg.h"
#include "kxstitch.h"
#include "flossscheme.h"
#include "floss.h"
#include "alphaselect.h"

#include <kdebug.h>

ImportImageDlg::ImportImageDlg(const Magick::Image& image, QWidget *parent, const char *name, bool modal)
  : ImportImage(parent,name,modal),
    m_pixmap(new QPixmap),
    m_useFractionals(false),
    m_orig(image),
    m_ignoreColor(false)
{
  m_originalSize = QSize(m_orig.columns(),m_orig.rows());
  QString caption = QString("Import Image - Image Size %1 x %2 pixels").arg(m_originalSize.width()).arg(m_originalSize.height());
  setCaption(caption);

  // get the users preferred pattern size in stitches
  double cc = KXSConfig().NewPattern_ClothCount;
  QString ccu = KXSConfig().NewPattern_ClothCountUnits;
  Import_ClothCount->setSuffix(ccu);
  Import_ClothCount->setRange(1,55,(ccu == "/cm")?0.01:1.0,false);
  Import_ClothCount->setValue(cc);
  m_preferredSize = QSize(KXSConfig().NewPattern_Width,KXSConfig().NewPattern_Height);
  int sw = m_preferredSize.width()*100/m_originalSize.width();
  int sh = m_preferredSize.height()*100/m_originalSize.height();
  int s = std::min(sw, sh); // pick smallest scale

  Import_FlossScheme->insertStringList(((KXStitchApplication*)kapp)->schemes());
  Configuration::setComboBoxItem(Import_FlossScheme,KXSConfig().NewPattern_FlossScheme);
  Import_PatternScale->setValue(s);
  Import_UseMaxColors->setChecked(KXSConfig().Import_UseMaxColors);
  Import_MaxColors->setValue(KXSConfig().Import_MaxColors);

  connect(Import_FlossScheme, SIGNAL(activated(int)), this, SLOT(schemeChanged(int)));
  connect(Import_UseMaxColors, SIGNAL(stateChanged(int)), this, SLOT(maxColorsChanged(int)));
  connect(Import_MaxColors, SIGNAL(valueChanged(int)), this, SLOT(maxColorsChanged(int)));
  connect(Import_ColorPicker, SIGNAL(clicked()), this, SLOT(pickColor()));
  connect(Import_ClothCount, SIGNAL(valueChanged(double)), this, SLOT(clothCountChanged(double)));
  connect(Import_PatternScale, SIGNAL(valueChanged(int)), this, SLOT(scaleChanged(int)));
  connect(Import_UseFractionals, SIGNAL(stateChanged(int)), this, SLOT(useFractionalsChanged(int)));
  connect(ResetButton, SIGNAL(clicked()), this, SLOT(resetImage()));
  connect(HelpButton, SIGNAL(clicked()), this, SLOT(contextHelp()));

  createImageMap();
  renderPixmap();
}

ImportImageDlg::~ImportImageDlg()
{
  delete m_pixmap;
}

void ImportImageDlg::resetImage()
{
  m_image = m_orig;
  renderPixmap();
}

void ImportImageDlg::schemeChanged(int)
{
  createImageMap();
  renderPixmap();
}

void ImportImageDlg::calcSizes()
{
  m_image = m_orig;
  m_preferredSize = m_originalSize*Import_PatternScale->value()/100;
  QSize imageSize = m_preferredSize;
  if (m_useFractionals)
  {
    // double the size of the image
    imageSize *= 2;
  }
  Magick::Geometry geom(imageSize.width(), imageSize.height());
  geom.percent(false);
  m_image.sample(geom);
  clothCountChanged(Import_ClothCount->value());
}

void ImportImageDlg::maxColorsChanged(int)
{
  killTimers();
  startTimer(500);
}

void ImportImageDlg::clothCountChanged(double n)
{
  double wi = m_preferredSize.width();
  double hi = m_preferredSize.height();
  QString suffix;
  QString units = KXSConfig().NewPattern_Units;
  if (units != "Stitches")
  {
    wi /= n;
    hi /= n;
    suffix = Import_ClothCount->suffix().right(2);
  }
  Import_PatternSize->setText(QString(i18n("%1%2 x %3%4")).arg(wi,0,'g',3).arg(suffix).arg(hi,0,'g',3).arg(suffix));
}

void ImportImageDlg::scaleChanged(int)
{
  killTimers();
  startTimer(500);
}

void ImportImageDlg::useFractionalsChanged(int s)
{
  m_useFractionals = (s == QButton::On);
  killTimers();
  startTimer(500);
}

void ImportImageDlg::createImageMap()
{
  FlossScheme *scheme = ((KXStitchApplication*)kapp)->scheme(Import_FlossScheme->currentText());
  m_map = *(scheme->createImageMap());
}

void ImportImageDlg::renderPixmap()
{
  QPixmap alpha = KGlobal::iconLoader()->loadIcon("alpha",KIcon::User);
  CropImageLabel->setCursor(waitCursor);
  calcSizes();
  m_image.modifyImage();
  m_pixmap->resize(m_image.columns(), m_image.rows());
  m_pixmap->fill();
  if (Import_UseMaxColors->isChecked())
  {
    // TODO add noise reduction routines to clean up converted images
    m_image.quantizeColorSpace(Magick::RGBColorspace);
    m_image.quantizeColors(Import_MaxColors->value());
    m_image.quantize();
  }
  m_image.map(m_map);
  m_image.modifyImage();

  QPainter p;
  p.begin(m_pixmap);
  p.drawTiledPixmap(m_pixmap->rect(),alpha);
  int w = m_image.columns();
  int h = m_image.rows();
  int pixelCount = w*h;

  QProgressDialog progress(i18n("Rendering preview..."), i18n("Cancel"), pixelCount, this, i18n("Progress"), true);
  progress.setMinimumDuration(100);

  Magick::Pixels cache(m_image);
  const Magick::PixelPacket *pixels = cache.getConst(0,0,w,h);
  for (int cy = 0 ; cy < h ; cy++)
  {
    kapp->processEvents();
    progress.setProgress(cy*w);
    if (progress.wasCancelled()) break;
    for (int cx = 0 ; cx < w ; cx++)
    {
      Magick::PixelPacket packet = *pixels++;
      if (!(packet.opacity))
      {
        if (!(m_ignoreColor && packet == m_ignoreColorValue))
        {
          QColor c(packet.red/257, packet.green/257, packet.blue/257);
          p.setPen(QPen(c));
          p.drawPoint(cx,cy);
        }
      }
    }
  }
  p.end();
  CropImageLabel->setPixmap(*m_pixmap);
  CropImageLabel->setCursor(arrowCursor);
}

void ImportImageDlg::accept()
{
  if (Import_SetDefault->isChecked())
  {
    KXSConfig().Import_UseMaxColors = Import_UseMaxColors->isChecked();
    KXSConfig().Import_MaxColors = Import_MaxColors->value();
    KXSConfig().writeSettings();
  }
  QDialog::accept();
}

void ImportImageDlg::contextHelp()
{
  kapp->invokeHelp("ImportImageDlg");
}

void ImportImageDlg::timerEvent(QTimerEvent *e)
{
  killTimers();
  renderPixmap();
}

void ImportImageDlg::pickColor()
{
  m_ignoreColor = Import_IgnoreColor->isChecked();
  kdDebug() << "In ImportImageDlg::pickColor()" << endl;
  if (m_ignoreColor)
  {
    m_alphaSelect = new AlphaSelect(CropImageLabel);
    connect(m_alphaSelect,SIGNAL(clicked(QPoint)),this,SLOT(selectColor(QPoint)));
    m_alphaSelect->show();
  }
  else
  {
    delete m_alphaSelect;
    m_alphaSelect = 0;
  }
}

void ImportImageDlg::selectColor(QPoint p)
{
  delete m_alphaSelect;
  m_alphaSelect = 0;
  QPixmap swatch(Import_IgnoreColorValue->size());
  int x = p.x()-((CropImageLabel->width()-m_pixmap->width())/2);
  int y = p.y()-((CropImageLabel->height()-m_pixmap->height())/2);
  m_ignoreColorValue = m_image.pixelColor(x, y);
  swatch.fill(QColor(m_ignoreColorValue.redQuantum()/257,m_ignoreColorValue.greenQuantum()/257,m_ignoreColorValue.blueQuantum()/257));
  Import_IgnoreColorValue->setPixmap(swatch);
  kdDebug() << "color at X:" << x << " Y:" << y << " is color " << m_ignoreColorValue.redQuantum() << "," << m_ignoreColorValue.greenQuantum() << "," << m_ignoreColorValue.blueQuantum() << endl;
  renderPixmap();
}
